/*
  regex.h - POSIX.2 compatible regexp interface

  Copyright (C) 2001, 2002 Ville Laurikari <vl@iki.fi>.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 2 (June
  1991) as published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef _REGEX_H
#define _REGEX_H 1

#ifdef __cplusplus
extern "C" {
#endif

#ifdef HAVE_WCHAR_H
#include <wchar.h>
#endif /* HAVE_WCHAR_H */

typedef int regoff_t;
typedef struct {
  size_t re_nsub;  /* Number of parenthesized subexpressions. */
  void *value;     /* For internal use only. */
} regex_t;

typedef struct {
  regoff_t rm_so;
  regoff_t rm_eo;
} regmatch_t;

/* The POSIX.2 regexp functions */
int regcomp(regex_t *preg, const char *regex, int cflags);
int regexec(const regex_t *preg, const char *string, size_t nmatch,
	    regmatch_t pmatch[], int eflags);
size_t regerror(int errcode, const regex_t *preg, char *errbuf,
		size_t errbuf_size);
void regfree(regex_t *preg);

/* Wide character versions (not in POSIX.2). */
int regwcomp(regex_t *preg, const wchar_t *regex, int cflags);
int regwexec(const regex_t *preg, const wchar_t *string, size_t nmatch,
	     regmatch_t pmatch[], int eflags);
size_t regwerror(int errcode, const regex_t *preg, wchar_t *errbuf,
		 size_t errbuf_size);

/* Versions with a maximum length argument and therefore the capability to
   handle null characters in the middle of the strings (not in POSIX.2). */
int regncomp(regex_t *preg, const char *regex, size_t n, int cflags);
int regwncomp(regex_t *preg, const wchar_t *regex, size_t n, int cflags);
int regnexec(const regex_t *preg, const char *string, size_t n,
	     size_t nmatch, regmatch_t pmatch[], int eflags);
int regwnexec(const regex_t *preg, const wchar_t *string, size_t n,
	      size_t nmatch, regmatch_t pmatch[], int eflags);

typedef enum {
  REG_OK = 0,           /* No error. */
  /* POSIX regcomp() return error codes.  (In the order listed in the
     standard.)  */
  REG_NOMATCH,          /* No match. */
  REG_BADPAT,           /* Invalid regexp. */
  REG_ECOLLATE,         /* Unknown collating element. */
  REG_ECTYPE,           /* Unknown character class name. */
  REG_EESCAPE,          /* Trailing backslash.  */
  REG_ESUBREG,
  REG_EBRACK,           /* "[]" imbalance */
  REG_EPAREN,           /* "\(\)" or "()" imbalance */
  REG_EBRACE,           /* "\{\}" or "{}" imbalance */
  REG_BADBR,            /* Invalid content of {} */
  REG_ERANGE,           /* Invalid use of range operator */
  REG_ESPACE,           /* Out of memory.  */
  REG_BADRPT
} reg_errcode_t;

/* The maximum number of iterations in a bound expression. */
#undef RE_DUP_MAX
#define RE_DUP_MAX 255

/* POSIX regcomp() flags. */
#define REG_EXTENDED    1
#define REG_ICASE       (REG_EXTENDED << 1)
#define REG_NEWLINE     (REG_ICASE << 1)
#define REG_NOSUB       (REG_NEWLINE << 1)

/* POSIX regexec() flags. */
#define REG_NOTBOL 1
#define REG_NOTEOL (REG_NOTBOL << 1)

#ifdef __cplusplus
}
#endif
#endif				/* _REGEX_H */

/* EOF */
